# +========================================================================+
# || module: CipUX::RPC::Client                                           ||
# ||                                                                      ||
# || Base Class for CipUX XML-RPC clients.                                ||
# ||                                                                      ||
# || Copyright (C) 2009 by Christian Kuelker                              ||
# ||                                                                      ||
# || License: GNU General Public License - GNU GPL version 2              ||
# ||          or (at your opinion) any later version.                     ||
# ||                                                                      ||
# +========================================================================+
#  ID:       $Id$
#  Revision: $Revision$
#  Head URL: $HeadURL$
#  Date:     $Date$
#  Source:   $Source$

package CipUX::RPC::Client;

use 5.008001;
use warnings;
use strict;
use Carp;
use Class::Std;
use Data::Dumper;
use English qw( -no_match_vars);
use Frontier::Client;
use Log::Log4perl qw(get_logger :levels);
use Readonly;

use base qw(CipUX);

{    # BEGIN CLASS

    # CONSTRUCTOR
    # DESTRUCTOR
    # MAIN
    # * RPC COMMANDS
    #   - rpc_ping
    #   - rpc_login
    #   - rpc_logout
    #   - rpc_ttl                   (rpc_intern)
    #   - rpc_cat_module_cache_size (rpc_intern)
    #   - rpc_rpc_intern_cache_size (rpc_intern)
    #   - rpc_task_cache_size       (rpc_intern)
    #   - rpc_flush                 (rpc_intern)
    #   - rpc_check_access
    # * RPC HELPER SUBS
    #   - xmlrpc

    use version; our $VERSION = qv('3.4.0.1');
    use re 'taint';    # Keep data captured by parens tainted
    delete @ENV{qw(PATH IFS CDPATH ENV BASH_ENV)};    # Make %ENV safer

    # +======================================================================+
    # || CONST                                                              ||
    # +======================================================================+
    Readonly::Scalar my $EMPTY_STRING => q{};
    Readonly::Scalar my $L4PCONF      => '/etc/cipux/log4perl.conf';

    # +======================================================================+
    # || OBJECT                                                             ||
    # +======================================================================+
    #my %storage_cfg : ATTR( :get<storage_cfg> );
    my %url_of :
        ATTR( :get<url_of> :set<url_of> :default('http://localhost:8001/RPC2') );
    my %client_of :
        ATTR( :init_arg<client_of> :get<client_of> :set<client_of> );
    my %version_of :
        ATTR( :init_arg<version_of>  :get<version_of> :set<version_of> );

    # +======================================================================+
    # || GLOBAL                                                             ||
    # +======================================================================+
    my $debug = 0;

    my $login    = 'dummy';
    my $password = undef;
    my $ticket   = 'dummy';

    my $rpc_ttl                   = 0;
    my $rpc_cat_module_cache_size = 0;
    my $rpc_rpc_intern_cache_size = 0;
    my $rpc_task_cache_size       = 0;

    # +======================================================================+
    # || CONSTRUCTOR                                                        ||
    # +======================================================================+
    sub BUILD {

        # +------------------------------------------------------------------+
        # | API
        my ( $self, $ident, $arg_r ) = @_;

        # +------------------------------------------------------------------+
        # | main

        # not needed, is already sub lcass
        #$cipux         = CipUX->new();

        # +------------------------------------------------------------------+
        # | API
        return;

    }

    # +======================================================================+
    # || DESTRUCTOR                                                         ||
    # +======================================================================+
    sub DEMOLISH {

        # +------------------------------------------------------------------+
        # | API
        my ( $self, $ident ) = @_;

        # +------------------------------------------------------------------+
        # | main
        #undef $cipux;
        delete $url_of{ ident $self};
        delete $client_of{ ident $self};
        delete $version_of{ ident $self};

        # +------------------------------------------------------------------+
        # | API
        return;

    }

    # +======================================================================+
    # || RPC methods                                                        ||
    # +======================================================================+
    sub rpc_selfpasswd {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        my $password
            = ( exists $arg_r->{password} )
            ? $self->l( $arg_r->{password} )
            : $self->perr('password');

        my $logger = get_logger(__PACKAGE__);
        $logger->debug('going to change the password');

        my $task = 'cipux_task_change_own_password_on_command_line';

        ( my $answer_hr, $ticket ) = $self->xmlrpc(
            {
                cmd      => $task,
                param_hr => {
                    object       => $login,
                    userPassword => $password,
                },

            }
        );

        my $status = $answer_hr->{status} || 'UNKNOWN';

        $logger->debug( 'the status of the XML-RPC server is: ',
            $status, "\n" );

        if ( $status eq 'FALSE' ) {
            my $msg
                = "EXCEPTION: Setting password via server $url_of{ident $self} failed!\n";
            $msg .= "($answer_hr->{msg})\n";

            croak $msg;
        }
        elsif ( $status eq 'UNKNOWN' ) {
            my $msg = 'EXCEPTION: Server status UNKNOWN' . "\n";
            croak $msg;
        }
        elsif ( $status eq 'TRUE' ) {
            return 1;
        }
        return 0;

    }

    sub rpc_ping {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        my $logger = get_logger(__PACKAGE__);
        $logger->debug('going to ping the XML-RPC server');

        ( my $answer_hr, $ticket ) = $self->xmlrpc(
            {
                login => 'dummy',
                cmd   => 'ping',
            }
        );

        my $status = $answer_hr->{status} || 'UNKNOWN';

        $logger->debug( 'the status of the XML-RPC server is: ',
            $status, "\n" );

        if ( $status eq 'FALSE' ) {
            my $msg
                = "EXCEPTION: Server $url_of{ident $self} not responding\n";

            croak $msg;
        }
        elsif ( $status eq 'UNKNOWN' ) {
            my $msg = 'EXCEPTION: Server status UNKNOWN' . "\n";
            croak $msg;
        }
        elsif ( $status eq 'TRUE' ) {
            return 1;
        }
        return 0;

    }

    sub rpc_login {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        my $lstr = "$client_of{ ident $self}: Enter login: ";
        $login = $self->l( $self->login_prompt( { prompt => $lstr } ) );

        #$logger->debug("Login: $login");

        my $pstr = "$client_of{ ident $self}: Enter password: ";
        $password
            = $self->lp( $self->password_prompt( { prompt => $pstr } ) );

        ( my $answer_hr, $ticket ) = $self->xmlrpc(
            {
                ticket   => 'dummy',
                cmd      => 'login',
                param_hr => { password => $password, },
            }
        );

        if ( $answer_hr->{status} eq 'TRUE' ) {
            return 1;
        }
        return 0;

    }

    sub rpc_logout {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        ( my $answer_hr, $ticket ) = $self->xmlrpc( { cmd => 'logout', } );

        if ( $answer_hr->{status} eq 'TRUE' ) {
            return 1;
        }
        return 0;

    }

    # rpc_intern
    # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
    # | SHELL RPC COMMANDS                                                   |
    # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+

    sub rpc_ttl {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        # +------------------------------------------------------------------
        # | main
        ( my $answer_hr, $ticket ) = $self->xmlrpc( { cmd => 'ttl', } );

        if ( $answer_hr->{status} ne 'TRUE' ) {
            croak "XML-RPC server ERRROR: $answer_hr->{msg}";
        }

        $rpc_ttl = $answer_hr->{cmdres_r}->{ttl};

        return $rpc_ttl;
    }

    sub rpc_cat_module_cache_size {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        # +------------------------------------------------------------------
        # | main
        ( my $answer_hr, $ticket ) = $self->xmlrpc(
            {
                cmd      => 'rpc_intern',
                param_hr => { subcmd => 'cat_module_cache_size', },
            }
        );

        if ( $answer_hr->{status} ne 'TRUE' ) {
            my $msg = 'XML-RPC server ERRROR:';
            $msg .= ' Can not retrieve cache cat_module cache size.';
            croak "$msg [$answer_hr->{msg}]";
        }

        $rpc_cat_module_cache_size
            = $answer_hr->{cmdres_r}->{cat_module_cache_size};

        return $rpc_cat_module_cache_size;

    }

    sub rpc_rpc_intern_cache_size {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        # +------------------------------------------------------------------
        # | main
        ( my $answer_hr, $ticket ) = $self->xmlrpc(
            {
                cmd      => 'rpc_intern',
                param_hr => { subcmd => 'rpc_intern_cache_size', },
            }
        );

        if ( $answer_hr->{status} ne 'TRUE' ) {
            my $msg = 'XML-RPC server ERRROR:';
            $msg .= ' Can not retrieve cache rpc_intern cache size.';
            croak "$msg [$answer_hr->{msg}]";
        }

        $rpc_rpc_intern_cache_size
            = $answer_hr->{cmdres_r}->{rpc_intern_cache_size};

        return $rpc_rpc_intern_cache_size;

    }

    sub rpc_task_cache_size {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        # +------------------------------------------------------------------
        # | main
        ( my $answer_hr, $ticket ) = $self->xmlrpc(
            {
                cmd      => 'rpc_intern',
                param_hr => { subcmd => 'task_cache_size', },
            }
        );

        if ( $answer_hr->{status} ne 'TRUE' ) {
            my $msg = 'XML-RPC server ERRROR:';
            $msg .= ' Can not retrieve cache task cache size.';
            croak "$msg [$answer_hr->{msg}]";
        }

        $rpc_task_cache_size = $answer_hr->{cmdres_r}->{task_cache_size};

        return $rpc_task_cache_size;

    }

    sub rpc_flush {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        # +------------------------------------------------------------------
        # | main
        ( my $answer_hr, $ticket ) = $self->xmlrpc(
            {
                cmd      => 'rpc_intern',
                param_hr => { subcmd => 'flush', },
            }
        );

        if ( $answer_hr->{status} ne 'TRUE' ) {
            my $msg = 'XML-RPC server ERRROR:';
            $msg .= ' Can not flush cache.';
            croak "$msg [$answer_hr->{msg}]";
        }

        return 1;

    }

    sub rpc_check_access {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        # rpc_info  | rpc_intern
        my $rpcmode
            = exists $arg_r->{rpcmode}
            ? $self->l( $arg_r->{rpcmode} )
            : $self->perr('rpcmode');

        # task   | rpc_intern | cat_module
        my $entity
            = exists $arg_r->{entity}
            ? $self->l( $arg_r->{entity} )
            : $self->perr('entity');

        # single | manifold
        my $scope
            = exists $arg_r->{scope}
            ? $self->l( $arg_r->{scope} )
            : $self->perr('scope');

        my $subcmd
            = exists $arg_r->{subcmd} ? $self->l( $arg_r->{subcmd} ) : $login;

        my $param_ar
            = exists $arg_r->{param_ar}
            ? $self->a( $arg_r->{param_ar} )
            : $self->perr('param_ar');

        # +------------------------------------------------------------------
        # | main

        # dispatch shell logic and rpc server logic
        # rpc shell     rpcmode        xml-rpc server command
        # ---------------------------------------------------
        # rbac      ->  rpc_info    -> rpc_info
        # rpc       ->  rpc_intern  -> rpc_intern
        my $cmd
            = ( $rpcmode eq 'rpc_info' ) ? 'rpc_info'
            : ( $rpcmode eq 'rpc_intern' ) ? 'rpc_intern'
            :   croak "CipUX shell Error: invalid mode [$rpcmode]\n";

        my $from = (
                   $subcmd eq 'user_task_access'
                or $subcmd eq 'user_task_access_survey'
                or $subcmd eq 'user_cat_module_access'
                or $subcmd eq 'user_cat_module_access_survey'
                or $subcmd eq 'user_rpc_intern_access'
                or $subcmd eq 'user_rpc_intern_access_survey'
        ) ? shift @{$param_ar} : undef;

        if ( $rpcmode ne 'rpc_intern' ) {
            $from = $login;
        }

        ( my $answer_hr, $ticket ) = $self->xmlrpc(
            {
                cmd      => $cmd,
                param_hr => {
                    subcmd  => $subcmd,
                    rpcmode => $rpcmode,
                    entity  => $entity,
                    scope   => $scope,
                    from    => $from,
                    to_ar   => $param_ar,
                },
            }
        );

        if ( $answer_hr->{status} ne 'TRUE' ) {
            croak "XML-RPC server ERRROR: $answer_hr->{msg}";
        }

        if (    exists $answer_hr->{cmdres_r}->{access_hr}
            and exists $answer_hr->{cmdres_r}->{from} )
        {

            my $r_hr = $answer_hr->{cmdres_r}->{access_hr};
            my $from = $answer_hr->{cmdres_r}->{from};

            if ( defined $r_hr and ref $r_hr eq 'HASH' ) {
                return ( $from, $r_hr );
            }

        }

        # +------------------------------------------------------------------
        # | API
        return ( undef, {} );

    }

    # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
    # | SHELL RPC GENERIC COMMAND                                            |
    # +~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+
    sub xmlrpc {

        # +------------------------------------------------------------------
        # | API
        my ( $self, $arg_r ) = @_;

        my $cmd
            = exists $arg_r->{cmd}
            ? $self->l( $arg_r->{cmd} )
            : $self->perr('cmd');

        my $tmp_login
            = exists $arg_r->{login}
            ? $self->l( $arg_r->{login} )
            : $login;

        my $tmp_ticket
            = exists $arg_r->{ticket}
            ? $self->l( $arg_r->{ticket} )
            : $ticket;

        my $param_hr
            = exists $arg_r->{param_hr}
            ? $self->h( $arg_r->{param_hr} )
            : {};

        # +------------------------------------------------------------------
        # | prepare

        my $logger = get_logger(__PACKAGE__);

        my $header_hr = {
            'cipux_version'  => "$VERSION",
            'client_name'    => "$client_of{ident $self}",
            'client_version' => "$version_of{ident $self}",
            'rpc_version'    => '2.0',
            'client_key'     => $EMPTY_STRING,
            'client_cred'    => $EMPTY_STRING,
            'gmt_time'       => time(),
        };

        my $pay_hr = {
            header_hr => $header_hr,
            login     => $tmp_login,
            ticket    => $tmp_ticket,
            cmd       => $cmd,
            param_hr  => $param_hr
        };

        my $socket = Frontier::Client->new( url => $url_of{ ident $self} );

        my $cmd_type = $cmd;
        if ( $cmd =~ m/^cipux_task/smx ) {
            $cmd_type = 'task';
        }

        my $answer_hr = $socket->call( $cmd_type, $pay_hr );

        # remember, if we got new ticket on the default channel
        if ( defined $answer_hr->{ticket}
            and $answer_hr->{ticket} ne $EMPTY_STRING )
        {
            $ticket = $answer_hr->{ticket};
        }

        # but if we got a ticket explicitly (e.g. at login): use it
        if ( ref( $answer_hr->{cmdres_r} ) eq 'HASH' ) {
            if ( defined $answer_hr->{cmdres_r}->{ticket}
                and $answer_hr->{cmdres_r}->{ticket} ne $EMPTY_STRING )
            {
                $ticket = $answer_hr->{cmdres_r}->{ticket};
            }
        } ## end if ( ref( $answer_hr->...

        return $answer_hr, $ticket;

    }

}

1;    # Magic true value required at end of module

__END__

=pod

=for stopwords CipUX XML-RPC Destructor xmlrpc webpage Kuelker TTL subcommands

=head1 NAME

CipUX::RPC::Client - XML-RPC client call library


=head1 VERSION

This document describes CipUX::RPC::Client version 3.4.0.1


=head1 SYNOPSIS

 use warnings;
 use strict;
 use CipUX::RPC::Client;
 use version; our $VERSION = qv('3.4.0.0');

 my $rpc = CipUX::RPC::Client->new({
               url_of=>'http://localhost:8001/RPC2',
               client_of=>'clientname', # name of client
               version_of=>'3.4.0.0', # version of client
          });

 if ( $rpc->rpc_ping() ) { print "OK\n" }else{ exit 1; }

 if ( $rpc->rpc_login() ) { print "OK\n" }else{ exit 1; }

 # probably do some other calls

 if ( $rpc->rpc_logout() ) { print "OK\n" }else{ exit 1; }

 exit 0;

=head1 DESCRIPTION


=head1 SUBROUTINES/METHODS

=head2 BUILD

Constructor (Not used at the moment)

=head2 DEMOLISH

Destructor

=head2 rpc_selfpasswd

Set own password.

  1|0 = $rpc->rpc_login( { password=>$password } );

=head2 rpc_ping

Test if the server is up.

  1|0 = $rpc->rpc_ping();

=head2 rpc_login

Perform a log in.

  1|0 = $rpc->rpc_login();

=head2 rpc_logout

Perform a log out.

  1|0 = $rpc->rpc_logout();

=head2 rpc_ttl

Returns the TTL via rpc_intern.

  $ttl = $rpc->rpc_ttl();

=head2 rpc_cat_module_cache_size

Returns the cache size via rpc_intern.

  $integer = $rpc->rpc_cat_module_cache_size();

=head2 rpc_rpc_intern_cache_size

Returns the cache size via rpc_intern.

  $integer = $rpc->rpc_rpc_intern_cache_size();

=head2 rpc_task_cache_size

Returns the cache size via rpc_intern.

  $integer = $rpc->rpc_task_cache_size();

=head2 rpc_flush

Flush the cache via rpc_intern.

  0|1 = $rpc->rpc_flush();

=head2 rpc_check_access

Aggregate several check access subcommands.


  ( $from, $r_hr ) = $rpc->rpc_check_access({
     rpcmode    => rpc_intern|rpc_info,
     entity     => task   | rpc_intern | cat_module
     scope      => single | manifold,
     subcommand => user_task_access
                   | user_task_access_survey
                   | user_cat_module_access
                   | user_cat_module_access_survey
                   | user_rpc_intern_access
                   | user_rpc_intern_access_survey

     param_ar   => (rest send as to_ar)
  });


=head2 xmlrpc

Helper subroutine to executing XML-RPC calls.


  ($answer_hr, $ticket) = $rpc->xmlrpc({
        cmd    => 'cipux_task_*|rpc_intern|...',  # mandatory
        login  => $login,
        ticket => $ticket,
        param_hr => {},
  });

=head1 DIAGNOSTICS

=over

=item C<< EXCEPTION: Setting password via server %url failed! >>


=item C<< EXCEPTION: Server %url not responding >>


=item C<< EXCEPTION: Server status UNKNOWN >>


=back


=head1 CONFIGURATION AND ENVIRONMENT

CipUX::RPC::Client requires no configuration files or environment
variables.


=head1 DEPENDENCIES

        Carp
        CipUX
        Class::Std
        Data::Dumper
        English
        Frontier::Client
        Log::Log4perl
        Readonly
        version


=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.


=head1 SEE ALSO

See the CipUX webpage and the manual at L<http://www.cipux.org>

See the mailing list L<http://sympa.cipworx.org/wws/info/cipux-devel>


=head1 AUTHOR

Christian Kuelker  E<lt>christian.kuelker@cipworx.orgE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 by Christian Kuelker. All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA

=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENSE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut
